/*
 * Accurate binary image selection from inaccurate user input
 *
 * Authors: Kartic Subr, Sylvain Paris, Cyril Soler, Jan Kautz
 *
 * Copyright (2013). All rights reserved.
 *
 * For details, see the paper:
 * Accurate binary image selection from inaccurate user input
 * Computer Graphics Forum 32, 2 (2013)
 * 
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for educational, research, and non-commercial
 * purposes, without fee and without a signed licensing agreement, is
 * hereby granted, provided that the above copyright notice and this
 * paragraph appear in all copies modifications, and distributions.
 *
 * Any commercial use or any redistribution of this software
 * requires a license.
 *
 * For further details, contact Kartic Subr (kartic@gmail.com)
 */

#ifndef __HIST_H__
#define __HIST_H__

#include <gsl/gsl_histogram.h>
#include <vector>
#include <cmath>

using std::vector ;

////////////////////////////////////////////////
// Wrapper for gsl_histogram
////////////////////////////////////////////////
class Histogram
{
    public:
	Histogram(int nBins, double min, double max) ;
	Histogram() ;
	~Histogram() ;
	
	void Init(int nBins, double min, double max) ;
	
	void Add(double  elem) ;
	double GetFreq(int bin) const ;
	int NBins() const ; 
	
    private: 
	    bool _initd ;
	    gsl_histogram * _h ;
	    int _n;
	    double _a, _b; 
};

////////////////////////////////////////////////
//  Multi-value Histogram class 
// eg, for a histogram of RGB, nVals=3
// Used by the DistHelper class to compute 
// Chisquared and EM distances on local histograms
////////////////////////////////////////////////
class MultivalHist
{
      public:
	MultivalHist(int nBins, double min, double max, int nVals) ;
	MultivalHist() ;
	~MultivalHist() ;
	
	void Init(int nBins, double min, double max, int nVals) ;
	
	void Add(const vector<double>& multielems) ;
	vector<double> GetFreq(int bin) const ;
	vector<double> GetvalHist(int channel) const ;

	int NBins() const ; 
	
	friend double ChiSqOld(const MultivalHist& h1, const MultivalHist& h2) ;
	friend double EMDdistOld(const MultivalHist& h1, const MultivalHist& h2) ;
	friend double Cosine(const MultivalHist& h1, const MultivalHist& h2) ;
    private: 
	    vector<gsl_histogram *>  _h ;
	    int _n, _nV;
	    double _a, _b; 
	    bool _inited ;
};

#endif //__HIST_H__